#include "TmeshLibrary.h"
#include "TrawMesh.h"

struct TmeshDesc {
	int		fMeshIndex;
	char*	fMeshName;
};

// Cylinder:
//
// Our cylinder has the height along the x-z plane, with radius of the cover being 1.0
// and height of cylinder is from 1.5 to -1.5
//
// Now: this says the bounding sphere has radius of sqrt(1.5*1.5 + 1*1) or about 1.8
//      We normalize the bounding sphere to between +1 and -1 (by dividing 1.8), which says, 
//      the bounding sphere has radius of 2.0. height of 1.5 height divided by 1.8 is about 
//      0.83, so the normalized cylinder has height going from z=-0.83 to z=+0.83, with radius
//      of the cover being around 1/1.8 or about 0.56.

static TmeshDesc sfAllMeshes[] =  {
		{eTigerMesh, 			"tiger.x"},
		{eCylinderMesh, 		"cylinder.x"},
		{eConeMesh, 			"cone.x"},
		{eTeapotMesh, 			"teapot.x"},
		{eShusuiMesh, 			"shusui.x"},
		{eCraftMesh, 			"bigship1.x"},
		{eCarMesh, 				"car.x"},
		{eSphereMesh, 			"sphere.x"},
		{eBoxMesh, 				"box.x"},

		//
		// add new entries above this last one
		//
		{eInvalidMesh,			""}
};

#define sfPathToMesh "./MeshFiles/"


TmeshLibrary::TmeshLibrary() 
{
	for (int i=0; i<sfMeshLibrarySize; i++)
		fMeshLibrary[i] = NULL;
}

TmeshLibrary::~TmeshLibrary()
{
	for (int i=0; i<sfMeshLibrarySize; i++) 
		if ( fMeshLibrary[i] != NULL )
			delete fMeshLibrary[i];
}

void TmeshLibrary::SetMeshToLibrary(
		TrawMesh *m, 
		int i)
{
	fMeshLibrary[i] = m;
}


TrawMesh *TmeshLibrary::GetMeshFromLibrary(int index) 
{
	#ifdef DEBUG_RELEASE
		CString msg;
		msg.Format("GetMeshFromLibrary(i=%d ptr=%x)", index, fMeshLibrary[index]);
		LogToFile(msg);
	#endif

	return fMeshLibrary[index];
}

void TmeshLibrary::InitializeMeshLibrary(
	LPDIRECT3DDEVICE9 pDevice,
	ID3DXMatrixStack *stack
)
{
	int i = 0;
	while (sfAllMeshes[i].fMeshIndex != eInvalidMesh) {
		int index = sfAllMeshes[i].fMeshIndex;
		fMeshLibrary[index] = new TrawMesh;
		if ( !fMeshLibrary[index]->LoadMesh(pDevice, stack, sfPathToMesh, sfAllMeshes[i].fMeshName) ) {
			CString msg;
			msg.Format("Loading of mesh:[%s] from [%s] failed!", sfAllMeshes[i].fMeshName, sfPathToMesh);
			WarningBox(msg);
			delete fMeshLibrary[index];
			fMeshLibrary[index] = NULL;
		}
			#ifdef DEBUG_RELEASE
				else {
					CString msg;
					msg.Format("Loading Mesh: Index(%d) Pointer(%x)", index, fMeshLibrary[index]);
					LogToFile(msg);
				}
			#endif
		i++;
	}
	#ifdef DEBUG_RELEASE
		PrintLibraryToLog("RightAfterBuild");
	#endif
}

#ifdef DEBUG_RELEASE
void TmeshLibrary::PrintLibraryToLog(const char *msg)
{	int i = 0;
	while (sfAllMeshes[i].fMeshIndex != eInvalidMesh) {
		int index = sfAllMeshes[i].fMeshIndex;
		CString m;
		m.Format("%s: fMeshLibrary(%x) Index[%d] Pointer(%x)", msg, &(fMeshLibrary), index, fMeshLibrary[index]);
		LogToFile(m);
		i++;
	}
}
#endif
