#include "TrawMeshSurface.h"

		const int TrawMeshSurface::kInvalidVertexIndex = -1;

TrawMeshSurface::TrawMeshSurface
(
	LPDIRECT3DDEVICE9 	device,
	int		    		numControlVertices
) 
		: fSelectedVertexIndex(kInvalidVertexIndex), fMesh(NULL)
{
	int i;

	fNumControlVertices = numControlVertices;
	fNumVertices = fNumControlVertices * fNumControlVertices;
	fNumFaces = (fNumControlVertices-1) * (fNumControlVertices-1) * 2;

	TmyVertex *v = new TmyVertex[fNumVertices];

	//
	//
	// the mesh geometry is defined in the range of ( -1 <->  1) in the x-z plane.
	// the vertices of the mesh are uniformlly distributed in this range.
	//
	fVertexDistance = 2.0f/float(fNumControlVertices-1.0f);
	int   nthRow = 0, nthColumn = 0;

	for (i=0; i<fNumVertices; i++) {
		v[i].fPoint.x = -1.0 + (nthColumn * fVertexDistance);
		v[i].fPoint.y = 0.0f;
		v[i].fPoint.z =  1.0 - (nthRow * fVertexDistance);
		nthColumn++;
		if (nthColumn>=fNumControlVertices) {
			nthRow++;
			nthColumn = 0;
		}

		v[i].fNormal.x = 0.0f;	// ComputeVertexNormal will do this right.
		v[i].fNormal.y = 1.0f;
		v[i].fNormal.z = 0.0f;

		v[i].fU = v[i].fU1 = (v[i].fPoint.x / 2.0f) + 0.5f;
		v[i].fV = v[i].fV1 = (v[i].fPoint.z / 2.0f) + 0.5f;
	}

	//
	// now fill in the strips of triangle (face) index
	// 
	// each strip has two rows of triangle
	//
	// Here are the observations:
	//
	// on nth strip: 
	//		lower row has index: 
	//				nth*fNumControlVertices, (nth*fNumControlVertices)+1, (nth+1)*fNumControlVertices
	//
	//      upper row has inex:
	//				(nth+1)*fNumControlVertices, (nth*fNumControlVertices)+1, ((nth+1)*fNumControlVertices)+1
	//
	// And each row have fNumControlVertices-1 number of triangles ...
	//
	WORD *indexBuffer = new WORD[fNumFaces*3];	// for index of vertices
	int indexCount=0;

	for (int strip = 0; strip < fNumControlVertices-1; strip++) {
		int lowV0 = strip*fNumControlVertices;
		int lowV1 = lowV0 + 1;
		int lowV2 = (strip+1)*fNumControlVertices;

		int hiV0 = lowV0 + fNumControlVertices;
		int hiV1 = lowV1;
		int hiV2 = hiV0 + 1;


		for (int rowTri=0; rowTri < fNumControlVertices-1; rowTri++) {
			//
			// lower row ...
			indexBuffer[indexCount++] = lowV0++;
			indexBuffer[indexCount++] = lowV1++;
			indexBuffer[indexCount++] = lowV2++;

			//
			// upper row
			indexBuffer[indexCount++] = hiV0++;
			indexBuffer[indexCount++] = hiV1++;
			indexBuffer[indexCount++] = hiV2++;
		}
	}

	//
	// Now load our mesh data into D3D ...
	//
	D3DXCreateMeshFVF(fNumFaces, fNumVertices, D3DXMESH_SYSTEMMEM, kMyVertexFormat, device, &fMesh);

	// 
	// 1. Get/Lock/Load the VertexBuffer
	void *pVBuffer;
	int vSize = sizeof(TmyVertex)*fNumVertices;
		fMesh->LockVertexBuffer( 0, &pVBuffer );
		memcpy(pVBuffer, (void *)v, vSize);
		fMesh->UnlockVertexBuffer();
	
	//
	// 2. Get/Lock/Load the IndexBuffer
	void *pIBuffer;
	int iSize = sizeof(WORD)*fNumFaces*3;
		fMesh->LockIndexBuffer( 0, &pIBuffer);
		memcpy(pIBuffer, (void *)indexBuffer, iSize);
		fMesh->UnlockIndexBuffer();

	//
	// Get Help to compute Normal ...
	//		Do this before redraw ...
	//		D3DXComputeNormals(fMesh, NULL);

	//
	//
	// Finally release our temporary memory ...
	//
	delete [] v;
	delete [] indexBuffer;
}

TrawMeshSurface::~TrawMeshSurface()
{
	SAFE_RELEASE(fMesh)
		// this is a macro defined in GraphicsAPI.h
		// check to make sure fMesh is not null
		// Release fMesh and then sets fMesh to Null
}
