#include "TrawMeshSurface.h"

//
// Selection of control vertex needs a little care since we
// are caching a copy of the vertex geometry locally in main
// memory, we must make sure the cached copy is consistent with
// the version inside the D3D pipeline ...
//


void TrawMeshSurface::LoadSelectedVertexFromD3D()
{
	if (fSelectedVertexIndex != kInvalidVertexIndex) {
		void *pVBuffer;
		fMesh->LockVertexBuffer( 0, &pVBuffer );
			TmyVertex *pCurrentV = &(((TmyVertex *)pVBuffer)[fSelectedVertexIndex]);
			fSelectedVertex = pCurrentV->fPoint;
		fMesh->UnlockVertexBuffer();
	}
}

void TrawMeshSurface::SetSelectedVertexToD3D()
{
	if (fSelectedVertexIndex != kInvalidVertexIndex) {
		void *pVBuffer;
		fMesh->LockVertexBuffer( 0, &pVBuffer );
			TmyVertex *pCurrentV = &(((TmyVertex *)pVBuffer)[fSelectedVertexIndex]);
			pCurrentV->fPoint = fSelectedVertex;
		fMesh->UnlockVertexBuffer();
	}
}

void TrawMeshSurface::SetToNoSelectedVertex() 
{
	//
	// check if we are currently selecting a valid vertex
	if (fSelectedVertexIndex != kInvalidVertexIndex) {
		// now we should update the index into the hardware ...
		SetSelectedVertexToD3D();
	}
	fSelectedVertexIndex = kInvalidVertexIndex;
}


void TrawMeshSurface::SelectVertex(int r, int c)
{
	//
	// 1. make sure we are not currently caching anything ...
	//
	SetSelectedVertexToD3D();

	fSelectedVertexIndex = IndexFromRowColumn(r, c);

	LoadSelectedVertexFromD3D();
}

void TrawMeshSurface::SelectedVertexPosition(D3DXVECTOR3 &p) const
{
	p = fSelectedVertex;
}

void TrawMeshSurface::SetSelectedVertexPosition(const D3DXVECTOR3 &p)
{
	fSelectedVertex = p;
	//
	// Notice we cache this, and only set into the Hardware before we draw ...
	//
}
