#include "../StdAfx.h"
#include "TrawMesh.h"
#include "../GraphicsAPI/Tmaterial.h"

#include "../GraphicsAPI/Tpoint.h"
#include "../GraphicsAPI/Tline.h"


//-----------------------------------------------------------------------------
bool TrawMesh::RayPickMesh(const D3DXVECTOR3 &rayO, const D3DXVECTOR3 &rayD)
{
    fStack->Push();
		SetUpD3DStack(fStack);

		D3DXMATRIX im, *m = fStack->GetTop();
		//
		// m will take us from object space to the world
		// our ray is in world, and we want to go to the object space
		//
		// 1. im is the inverse of m, use to transform point
		// 
		// 2. We do not need the inverse transpose here since we are _NOT_ transforming
		//       normal of a plane, we are simply transforming direction.
		//
		D3DXVECTOR3 o, dir;

		D3DXMatrixInverse(&im, NULL, m);	
		
		D3DXVec3TransformCoord(&o, &rayO, &im);
		D3DXVec3TransformNormal(&dir, &rayD, &im);

		//
		// Now get D3DX help ...
		BOOL hit = FALSE;
		DWORD face;
		float bary1, bary2, dist;
		D3DXIntersect(fMesh, &o, &dir, &hit, &face, &bary1, &bary2, &dist, NULL, NULL);

		#ifdef PICK_RAW_MESH_DEBUG
		if (hit == TRUE) {
			D3DXVECTOR3 mo; // this is the o + dir ray hitting the x/z plane
			float t;
			#define VERY_SMALL 0.000001
			if (fabs(dir.y) > VERY_SMALL) {
				// if dir is not parallel to the x/z plane
				t = -o.y/dir.y;
			} else {
				if (fabs(dir.x) > VERY_SMALL) {
					t = -o.x/dir.x;
				} else {
					t = -o.z/dir.z;
				}
			}
			mo = o + dir * t;
			extern Tpoint sfWorldSph;
			sfWorldSph.SetPointSize(2);
			sfWorldSph.DisableLighting();
			Tmaterial &smt = sfWorldSph.GetMaterial();
			smt.SetEmissive(1, 0, 0);
			sfWorldSph.SetLocationTo(mo.x, mo.y, mo.z);
			
			D3DXVECTOR3 p1 = mo - dir * 10;
			D3DXVECTOR3 p2 = mo + dir * 10;
			extern Tline sfWorldLine;
			sfWorldLine.DisableLighting();
			Tmaterial &lmt = sfWorldLine.GetMaterial();
			lmt.SetEmissive(1, 0, 0);
			sfWorldLine.SetFirstVertex(p1.x, p1.y, p1.z);
			sfWorldLine.SetSecondVertex(p2.x, p2.y, p2.z);

			extern TrawMesh *sfPickedMesh;
			sfPickedMesh = this;
		}
		#endif
		
	fStack->Pop();

	return (hit==TRUE);
}

