#ifndef T2DBOUND_H
#define T2DBOUND_H

#include "GraphicsAPI.h"


class T2dBound {
	private:
		D3DXVECTOR3 fMin, fMax;

		#define UpdateRange(_pt_, _axis_) {				\
			if (_pt_.##_axis_## > fMax.##_axis_##)		\
				fMax.##_axis_## = _pt_.##_axis_##;		\
			if (_pt_.##_axis_## < fMin.##_axis_##)		\
					fMin.##_axis_## = _pt_.##_axis_##;	\
		}

		#define PtInRange(_pt_, _obj_, _axis_) 				\
					((_pt_.##_axis_## >= ##_obj_##.fMin.##_axis_##) && (_pt_.##_axis_## <= ##_obj_##.fMax.##_axis_##))
		
	public:

		T2dBound();
		T2dBound(float xMin, float xMax, float yMin, float yMax);

		const D3DXVECTOR3& MinPosition() const;
		const D3DXVECTOR3& MaxPosition() const;

		float Width() const;
		float Height() const;

		void SetBound(float xMin, float xMax, float yMin, float yMax);

			//
			// give a new width and height without
			// moving the bound ...
		void SetBoundSizeTo(float w, float h);

			//
			// Move the entire bound by (dx,dy)
		void MoveBoundBy(float dx, float dy);

			//
			// Move the min pt to (x,y) maintining w/h
			// of the bound
		void SetBoundMinTo(float x, float y);

		void ResetBound();
			// set to an invalid bound

		bool ValidBound() const;

		void UpdateBound(const D3DXVECTOR3 pt);
		void UpdateBound(const T2dBound &b);

		bool BoundIntersect(const T2dBound &b) const;
		bool PointInBound(const D3DXVECTOR3 pt) const;

		void GetBoundaryPoints(D3DXVECTOR3 pt[4]) const;

};

#endif
