#ifndef TCAMERA_H
#define TCAMERA_H

#include "GraphicsAPI.h"
#include "TgraphicsObject.h"

#define CameraAttr(_Name_, _type_, _rtype_)				\
			private:									\
				_type_ f##_Name_##;						\
			public:										\
				void Set##_Name_(_type_ v);				\
				_rtype_ Get##_Name_() const;


#define ImplementCameraAttr(_Name_, _type_, _rtype_)	\
			void Tcamera::Set##_Name_(_type_ v) {		\
					f##_Name_## = v;					\
				}										\
			_rtype_ Tcamera::Get##_Name_() const {		\
					return f##_Name_##;					\
				}										

#define ImplementViewAttr(_Name_, _type_, _rtype_)		\
			void Tcamera::Set##_Name_(_type_ v) {		\
					f##_Name_## = v;					\
					InitializeUVW();					\
				}										\
			_rtype_ Tcamera::Get##_Name_() const {		\
					return f##_Name_##;					\
				}

//
class Tcamera : public TgraphicsObject {
		CameraAttr(FieldOfView, float, float)
		CameraAttr(AspectRatio, float, float)
		CameraAttr(Near,		float, float)
		CameraAttr(Far,			float, float)

		CameraAttr(EyePosition, D3DXVECTOR3, const D3DXVECTOR3&)
		CameraAttr(LookAt,		D3DXVECTOR3, const D3DXVECTOR3&)
		CameraAttr(UpVector,	D3DXVECTOR3, const D3DXVECTOR3&)

		BoolObjAttr(MoveEyePosition)
		BoolObjAttr(ShowRailGrid)

	private:
		D3DXVECTOR3	fU, 	// this is the unit vector in the up direction
					fV, 	// this is the unit vector in the viewing direction
					fW;		// this is the unit vector perpendicular to UV plane
		float		fViewDistance;	// distance between the eye and at point

		void		UpdatePosition();
						// Move eye point or at appoint according to fV and fViewDistance
						//

		void		InitializeUVW();
						// Iniitialize UVW based on fEyePosition, fLookAt, and fUpVector
						//

	protected:

		//
		// from TgraphicsObject
		// 
		virtual void DrawObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9);

	public:

		Tcamera();
		Tcamera(
				D3DXVECTOR3&,	// Eye Position
				D3DXVECTOR3&,	// Look At Position
				D3DXVECTOR3&,	// Up vector
				float, 			// FOV
				float, 			// Aspect Ratio
				float, 			// Near
				float);			// Far

		void 	DefineViewAndProjection(LPDIRECT3DDEVICE9);

				//
				// Support viewing distance
		float	GetViewDistance();
		void	SetViewDistance(float d);

				//
				// Rotating/manipulating the camera (either Eye or At)
		void	RotateCameraX(float);
		void	RotateCameraY(float);
		void	SetCameraTrack(float dx, float dy);

		void	DrawRailGrid(Td3dDrawable&);
				// the circular grid for rotating the camera

		//
		// These two methods will overrdie TgraphicsObject's
		// The reason we want to override this two functions is
		// that depending on fMoveEyePosition setting, we may
		// want to responde by setting eye/at position
		virtual void SetLocationTo(float x, float y, float z);
		virtual const D3DXVECTOR3& GetLocation() const;
};

#endif
