#include "../Td3dDrawable.h"
#include "Tcamera.h"
#include "Tcircle.h"
#include "Tcamera.h"
#include "TaxisFrame.h"
#include "Tline.h"
#include "Tmesh.h"



void
Tcamera::DrawRailGrid(Td3dDrawable &d)
{
	float R = GetViewDistance();

	const int kNumSlices = 6;	// on each of the hemi-sphere

	// 1. the horizontal slices
	float theta;
	float delta = M_PI/(2*kNumSlices);

	Tcircle c;
	c.SetFillMode(D3DFILL_WIREFRAME);
	c.SetDrawColor(D3DXCOLOR(0.3, 0.9, 0.9, 0));
	c.SetRadius(R);
	c.SetLocationTo(fLookAt.x, fLookAt.y, fLookAt.z);
	c.DrawObject(d);
	for (int i = 1; i<=kNumSlices; i++) {
		theta = i * delta;
		float y = R * sin(theta);
		float r = R * cos(theta);
		c.SetLocationTo(fLookAt.x, fLookAt.y+y, fLookAt.z);
		c.SetRadius(r);
		c.DrawObject(d);

		c.SetLocationTo(fLookAt.x, fLookAt.y-y, fLookAt.z);
		c.DrawObject(d);
	}

}


void
Tcamera::DrawObjectGeom(
	Td3dDrawable &d,
	LPDIRECT3DDEVICE9 device
)
{
	// 1. Pop off the Xform information from the stack
	ID3DXMatrixStack *stack = d.Obj2WldStack();
		
		//
		// pop-off stack from TgraphicsObject of Tcamera ...
		// we will build everything from scratch in here ...
		stack->Pop();
		stack->Push();	// this pop/push pairs un-done what TgraphicsObject::DrawObject has done

			Tmesh sph;
			sph.SetMeshIndex(eConeMesh);
			sph.SetDrawColor(D3DXCOLOR(1, 0, 0, 0));
			TxformInfo &xt = sph.GetXformInfo();
			D3DXMATRIX m;
			D3DXMatrixIdentity(&m);
			m._11 = fW.x;
			m._12 = fW.y;
			m._13 = fW.z;
			m._21 = fU.x;
			m._22 = fU.y;
			m._23 = fU.z;
			m._31 = fV.x;
			m._32 = fV.y;
			m._33 = fV.z;
			xt.SetOrientation(m);
			xt.SetScale(1, 1, 2);
			xt.UpdateRotation(0, M_PI, 0);
			xt.SetMovement(fEyePosition.x, fEyePosition.y, fEyePosition.z);
			sph.EnableShowAxisFrame();
			sph.DrawObject(d);

			TaxisFrame af;
			af.SetScale(2);
			af.SetLocationTo(fEyePosition.x, fEyePosition.y+0.2, fEyePosition.z);
			af.DrawObject(d);

			TaxisFrame f;
			D3DXVECTOR3 p = fLookAt;
			f.SetLocationTo(fLookAt.x, fLookAt.y, fLookAt.z);

			Tline v;
			v.SetFirstVertex(fLookAt.x, fLookAt.y, fLookAt.z);
			v.SetSecondVertex(fEyePosition.x, fEyePosition.y, fEyePosition.z);
			
			f.DrawObject(d);
			v.DrawObject(d);

			if (ShowRailGridIsEnabled())
				DrawRailGrid(d);
	
}
