#include "Tfillable.h"
#include "../Td3dDrawable.h"
#include "../Collections/Ttexture.h"

Tfillable::Tfillable()
{
	InitializeFillableObject();
}

ImplementSimpleObjAttr(Tfillable, WORD, FillMode)
ImplementSimpleObjAttr(Tfillable, int,  TextureIndex)
ImplementBoolObjAttr(Tfillable, AlphaBlending)
ImplementBoolObjAttr(Tfillable, ShowNormal)
ImplementBoolObjAttr(Tfillable, ShowTexture)

void Tfillable::InitializeFillableObject()
{
	fFillMode = D3DFILL_SOLID;
	fTextureIndex = -1;
	fAlphaBlending = false;
	fShowNormal = false;
	fShowTexture = false;
}


void Tfillable::SetObjectD3DAttributes(
	Td3dDrawable &d)
{
	//
	// Do what parent have to do ...
	//
	TdrawableObject::SetObjectD3DAttributes(d);
	d.D3DDevice()->SetRenderState(D3DRS_FILLMODE,    fFillMode);

	//
	//  Enable the file texture if defined ...
	//
	if ( ShowTextureIsEnabled() && (fTextureIndex >= 0) ){
		Ttexture *texture = d.fTextureLibrary.GetTextureFromLibrary(fTextureIndex);
		if (texture != NULL) {
			texture->ActivateTexture();
		}
	}

	if (AlphaBlendingIsEnabled()) {
		// 1. enable the blending
		d.D3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

		// 2. define what we want to do ...
		//
		//   OurColor * (AlphaFromTexture) + (FrameBufferColor * (1-AlphaFromTexture) )
		//
		d.D3DDevice()->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		d.D3DDevice()->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	}

}


void Tfillable::UnsetObjectD3DAttributes(Td3dDrawable &d)
{
	TdrawableObject::UnsetObjectD3DAttributes(d);

	//
	//  disable the file texture if defined ...
	//
	if ( ShowTextureIsEnabled() && (fTextureIndex >= 0) ) {
		Ttexture *texture = d.fTextureLibrary.GetTextureFromLibrary(fTextureIndex);
		if (texture != NULL) {
			texture->DeActivateTexture();

		}
	}
	
	if (AlphaBlendingIsEnabled()) {
		// disable alpha blending
		d.D3DDevice()->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	}
}
