#ifndef TGENERAL_ROTATION_H
#define TGENERAL_ROTATION_H

#include "GraphicsAPI.h"

class TgeneralRotation { 
		
	private:
		D3DXMATRIX			fRotationMatrix;
		D3DXVECTOR3			fRotation;				// current rotated amount
													// this will show up at the slider bar
						// 
						// fRotation encodes the entire rotation
						// aggrigage of the entire rotation history
						//
						//    e.g.: if user specify rotation: rx1, ry1, rx2, ry2
						//
						//
						//  by the time we see ry2:
						//
						//  	fRotation.y() = ry1
						//  	fRotation.x() = rx1 + rx2
						//
						//  	fRotationMatrix is the combine rotation of: rx1 * ry1 * rx2
						//
						//  notice: 
						//
						//  	fRotation remembers the total amount rotated in each axis
						//  			  with fRotation alone WE CANNOT reconstruct the 
						//  			  rotation history
						//
						//		fRotationMatrix: only remembers transformation required to
						//					rotate the original frame to the current orientation
						//  	

	public:

		TgeneralRotation();

		float x() const;
		float y() const;
		float z() const;
		const D3DXVECTOR3& Rotation() const;

		void ResetRotation();

		void SetRotation(float);			// force the amount (in radians)
		void SetRotation(const D3DXVECTOR3 &);
		void UpdateRotationX(float dx); 	// new amout of x rotation (in radians)
		void UpdateRotationY(float dy); 	// new amout of y rotation (in radians)
		void UpdateRotationZ(float dz); 	// new amout of x rotation (in radians)
		void UpdateRotation(const D3DXVECTOR3 &r);
		void SetOrientation(CONST D3DXMATRIX &m);
		const D3DXMATRIX& GetRotationMatrix() const { return fRotationMatrix; }

		void SetToStack(ID3DXMatrixStack *) const;
};

#endif
