#include "../StdAfx.h"
#include "../Td3dDrawable.h"

#include "TgraphicsObject.h"
#include "TaxisFrame.h"


TgraphicsObject::~TgraphicsObject()
{
}

TgraphicsObject::TgraphicsObject()
{
	InitializeGraphicsObject();
}

ImplementCompoundObjAttr(TgraphicsObject, TxformInfo, XformInfo)
ImplementCompoundObjAttr(TgraphicsObject, D3DXVECTOR3, Velocity)
ImplementCompoundObjAttr(TgraphicsObject, Tmaterial, Material)
ImplementBoolObjAttr(TgraphicsObject, HiLight)
ImplementBoolObjAttr(TgraphicsObject, Visibility)
ImplementBoolObjAttr(TgraphicsObject, ShowAxisFrame)
ImplementBoolObjAttr(TgraphicsObject, Lighting)


	
bool TgraphicsObject::ShouldDrawObject() const { return VisibilityIsEnabled(); }
void TgraphicsObject::SetObjectD3DAttributes(Td3dDrawable &d) 
{
}
void TgraphicsObject::UnsetObjectD3DAttributes( Td3dDrawable&) {}
void TgraphicsObject::DrawObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9) {}
	
void TgraphicsObject::SetLocationTo(float x, float y, float z) { fXformInfo.SetMovement(x, y, z); }
void TgraphicsObject::SetScale(float x, float y, float z) { fXformInfo.SetScale(x, y, z); }
void TgraphicsObject::SetScale(float s) { fXformInfo.SetScale(s); }
void TgraphicsObject::SetRotation(float r) { fXformInfo.SetRotation(r); }
void TgraphicsObject::SetRotation(float x, float y, float z) { fXformInfo.SetRotation(x, y, z); }

const D3DXVECTOR3& TgraphicsObject::GetLocation() const { return fXformInfo.Movement(); }
const D3DXVECTOR3& TgraphicsObject::GetScale() const { return fXformInfo.Scale(); }
const D3DXVECTOR3& TgraphicsObject::GetRotation() const { return fXformInfo.Rotation(); }

				
//
void TgraphicsObject::SetObjectD3DTransformation(ID3DXMatrixStack *stack) const {
	fXformInfo.SetUpD3DXformStack(stack);
}

	//  point size
void TgraphicsObject::SetPointSize(const float s) {}
void TgraphicsObject::SetFillMode(const WORD) {}
void TgraphicsObject::SetShadedMode(const WORD) {}
void TgraphicsObject::SetDrawColor(const D3DXCOLOR&) {}
void TgraphicsObject::SetTextureIndex(const int i) {}
bool TgraphicsObject::RayPickObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9, const D3DXVECTOR3&, const D3DXVECTOR3&) {
				return false; }


void
TgraphicsObject::UpdateLocation()
{
	fXformInfo.UpdateMovement(fVelocity.x, fVelocity.y, fVelocity.z);
}

void
TgraphicsObject::InitializeGraphicsObject()
{
	fHiLight = false;
	fVisibility = true;
	fShowAxisFrame = false;
	fLighting = true;
	fVelocity.x = fVelocity.y = fVelocity.z = 0.0f;
}

void 
TgraphicsObject::EchoHighLight(
	LPDIRECT3DDEVICE9
) const 
{
	// e.g. we can draw a rectange around 
	// current object.
}



void TgraphicsObject::DrawObject(
	Td3dDrawable &drawable
)
{

	if ( ShouldDrawObject() ) {

		ID3DXMatrixStack *toWorldStack = drawable.Obj2WldStack();
		LPDIRECT3DDEVICE9 d3dDevice = drawable.D3DDevice();


		//
		// save current stack ...
		//
		toWorldStack->Push();


			SetObjectD3DTransformation(toWorldStack);

			//
			// transfer the xform info to the system matrix
			//
			d3dDevice->SetTransform(D3DTS_WORLD, toWorldStack->GetTop());

			SetObjectD3DAttributes(drawable);

				DrawObjectGeom(drawable, d3dDevice);

			UnsetObjectD3DAttributes(drawable);
			
			// if (fHiLighted)
			//	EchoHighLight();


			if (ShowAxisFrameIsEnabled()) {
					TaxisFrame ax;
					ax.DrawObject(drawable);
			}
		//
		// Now restore the transforamtion stack
		//
		toWorldStack->Pop();

		d3dDevice->SetTransform(D3DTS_WORLD, toWorldStack->GetTop());
	}
}


bool TgraphicsObject::RayPickObject(
	Td3dDrawable &drawable,
	const D3DXVECTOR3 &rayO,
	const D3DXVECTOR3 &rayD
)
{
	bool picked = false;

	if ( ShouldDrawObject() ) {

		ID3DXMatrixStack *toWorldStack = drawable.Obj2WldStack();
		LPDIRECT3DDEVICE9 d3dDevice = drawable.D3DDevice();


		//
		// save current stack ...
		//
		toWorldStack->Push();


			SetObjectD3DTransformation(toWorldStack);

			//
			// transfer the xform info to the system matrix
			//
			D3DXMATRIX *m = toWorldStack->GetTop();
			d3dDevice->SetTransform(D3DTS_WORLD, m);


			picked = RayPickObjectGeom(drawable, d3dDevice, rayO, rayD);

		//
		// Now restore the transforamtion stack
		//
		toWorldStack->Pop();

		d3dDevice->SetTransform(D3DTS_WORLD, toWorldStack->GetTop());
	}

	return picked;
}
