#ifndef TMATERIAL_H
#define TMATERIAL_H

#include "GraphicsAPI.h"

#define SetMaterialProperty(materialType) 								\
	void Set##materialType##(float v)	{								\
		fm.##materialType##.r = v;										\
		fm.##materialType##.g = v;										\
		fm.##materialType##.b = v;										\
		fm.##materialType##.a = v;										\
	}																	\
	void Set##materialType##(float r, float g, float b, float a=1.0) {	\
		fm.##materialType##.r = r;										\
		fm.##materialType##.g = g;										\
		fm.##materialType##.b = b;										\
		fm.##materialType##.a = a;										\
	}																	\
	void Set##materialType##(const D3DXCOLOR &v) {						\
		fm.##materialType##.r = v.r;									\
		fm.##materialType##.g = v.g;									\
		fm.##materialType##.b = v.b;									\
		fm.##materialType##.a = v.a;									\
	}																	\
	void Get##materialType##(D3DXCOLOR &v) const {						\
		v.r = fm.##materialType##.r;									\
		v.g = fm.##materialType##.g;									\
		v.b = fm.##materialType##.b;									\
		v.a = fm.##materialType##.a;									\
	}


//
// material property for D3DX: We wrap around D3DMATERIAL9 structure providing a collection
// of accessing methods.
//
class Tmaterial {
	//
	private:
		D3DMATERIAL9	fm;		// this is the material
								// fm has: 	Diffuse		(D3DCOLORVALUE)
								// 			Ambient		(D3DCOLORVALUE)
								// 			Specular	(D3DCOLORVALUE)
								// 			Emissive	(D3DCOLORVALUE)
								// 			power 		(float)

	public:
		Tmaterial(float e, float a, float d, float s, float  p) {
			SetEmissive(e);
			SetAmbient(a);
			SetDiffuse(d);
			SetSpecular(s);
			SetPower(p);
		}

		Tmaterial() {
			//
			//
			// nice shade of Husky purple ...
			//
			SetAmbient(0.13f, 0.065f, 0.195f, 1.0f);
			//
			//
			// grey-ish  dull color otherwise ...
			// //
			SetEmissive(0.1f);
			SetDiffuse(0.8f);
			SetSpecular(0.2f);
			SetPower(1.0f);
		}

		//
		// define the setting and accessing functions
		//
		SetMaterialProperty(Emissive)
		SetMaterialProperty(Ambient)
		SetMaterialProperty(Diffuse)
		SetMaterialProperty(Specular)

		void  SetPower(float s) { fm.Power = s; }
		float GetPower() { return fm.Power; }

		void SetD3DMaterial(LPDIRECT3DDEVICE9 device) const {
			device->SetMaterial(&fm);
		}
};
#endif
