#include "Trectangle.h"
#include "../Td3dDrawable.h"


Trectangle::Trectangle()
{
	InitializeRectangleObject();
}

Trectangle::Trectangle(float scale)
{
	InitializeRectangleObject();
	fXformInfo.SetScale(scale);
}

void Trectangle::InitializeRectangleObject()
{
	int count = 0;
	fV[count].fPoint = D3DXVECTOR3(-0.5f, 0, 0.5f);
	fV[count].fNormal = D3DXVECTOR3(0, 1, 0);
	fV[count].fU = fV[count].fU1 = fV[count].fPoint.x+0.5f;
	fV[count].fV = fV[count].fV1 = fV[count].fPoint.z+0.5f;
	
	count++;
	fV[count].fPoint = D3DXVECTOR3( 0.5f, 0,  0.5f);
	fV[count].fNormal = D3DXVECTOR3(0, 1, 0);
	fV[count].fU = fV[count].fU1 = fV[count].fPoint.x+0.5f;
	fV[count].fV = fV[count].fV1 = fV[count].fPoint.z+0.5f;
	
	count++;
	fV[count].fPoint = D3DXVECTOR3(-0.5f, 0, -0.5f);
	fV[count].fNormal = D3DXVECTOR3(0, 1, 0);
	fV[count].fU = fV[count].fU1 = fV[count].fPoint.x+0.5f;
	fV[count].fV = fV[count].fV1 = fV[count].fPoint.z+0.5f;
	
	count++;
	fV[count].fPoint = D3DXVECTOR3( 0.5f, 0, -0.5f);
	fV[count].fNormal = D3DXVECTOR3(0, 1, 0);
	fV[count].fU = fV[count].fU1 = fV[count].fPoint.x+0.5f;
	fV[count].fV = fV[count].fV1 = fV[count].fPoint.z+0.5f;
}


void
Trectangle::GetPlaneEquationAndVertices(
	D3DXVECTOR3 &n,
	float    &d,
	D3DXVECTOR3 &p0,
	D3DXVECTOR3 &p1,
	D3DXVECTOR3 &p2,
	D3DXVECTOR3 &p3

)
	//
	// Equation returned is n dot p = d
	//
{
	//
	// Vertices of the plane
	p0 = D3DXVECTOR3(0.5, 0, -0.5);
	p1 = D3DXVECTOR3(0.5, 0, 0.5);
	p2 = D3DXVECTOR3(-0.5, 0, 0.5);
	p3 = D3DXVECTOR3(-0.5, 0, -0.5);

	const D3DXMATRIX &r = fXformInfo.GetRotationMatrix();
	D3DXMATRIX tm, sm, m;
	const D3DXVECTOR3 &t = GetLocation();
	const D3DXVECTOR3 &s = GetScale();
	D3DXMatrixTranslation(&tm, t.x, t.y, t.z);
	D3DXMatrixScaling(&sm, s.x, s.y, s.z);
	D3DXMatrixMultiply(&m, &sm, &r);
	D3DXMatrixMultiply(&m, &m, &tm);


	D3DXVec3TransformCoord(&p0, &p0, &m);	// now transform the points
	D3DXVec3TransformCoord(&p1, &p1, &m);
	D3DXVec3TransformCoord(&p2, &p2, &m);
	D3DXVec3TransformCoord(&p3, &p3, &m);

	D3DXVECTOR3 v2 = p2 - p0;
	D3DXVECTOR3 v1 = p1 - p0;

	D3DXVec3Cross(&n, &v2, &v1);
	D3DXVec3Normalize(&n, &n);
	d = -D3DXVec3Dot(&n, &p0);
}

void Trectangle::DrawObjectGeom(
	Td3dDrawable 	   &drawable,
	LPDIRECT3DDEVICE9  d3dDevice
) 
{

	if (fXformInfo.ScaleX() < 0)
		return;

	d3dDevice->SetFVF(kMyVertexFormat);
	d3dDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, (CONST void *)fV, sizeof(TmyVertex) );
}
