#include "TxformInfo.h"


TxformInfo::TxformInfo() {
	ResetTransformation();
}

void TxformInfo::ResetTransformation() {
	fMove.x = fMove.y = fMove.z = 0;
	fScale.x = fScale.y = fScale.z = 1;
	fRotation.ResetRotation();
}

void TxformInfo::SetUpD3DXformStack(ID3DXMatrixStack *stack) const {
	stack->TranslateLocal(fMove.x, fMove.y, fMove.z);
	fRotation.SetToStack(stack);
	stack->ScaleLocal(fScale.x, fScale.y, fScale.z);
}

float TxformInfo::MoveX() const { return fMove.x; }
float TxformInfo::MoveY() const { return fMove.y; }
float TxformInfo::MoveZ() const { return fMove.z; }
const D3DXVECTOR3& TxformInfo::Movement() const { return fMove; }

float TxformInfo::ScaleX() const { return fScale.x; }
float TxformInfo::ScaleY() const { return fScale.y; }
float TxformInfo::ScaleZ() const { return fScale.z; }
const D3DXVECTOR3& TxformInfo::Scale() const { return fScale; }

float TxformInfo::RotateX() const { return fRotation.x(); }
float TxformInfo::RotateY() const { return fRotation.y(); }
float TxformInfo::RotateZ() const { return fRotation.z(); }
const D3DXVECTOR3& TxformInfo::Rotation() const { return fRotation.Rotation(); }

void TxformInfo::SetMovement(const D3DXVECTOR3 &d) {
		fMove = d; }
void TxformInfo::SetMovement(float x, float y, float z) {
		fMove.x = x;
		fMove.y = y;
		fMove.z = z; }
void TxformInfo::UpdateMovement(float x, float y, float z) {
		fMove.x += x;
		fMove.y += y; 
		fMove.z += z; }

void TxformInfo::SetScale(const D3DXVECTOR3 &s) { 
		fScale = s; }
void TxformInfo::SetScale(float s) { 
		fScale.x = s;
		fScale.y = s; 
		fScale.z = s; }
void TxformInfo::SetScale(float x, float y, float z) {
		fScale.x = x;
		fScale.y = y;
		fScale.z = z; }
void TxformInfo::UpdateScale(float s) {
		fScale.x *= s;
		fScale.y *= s;
		fScale.z *= s; }
void TxformInfo::UpdateScale(float x, float y, float z) {
		fScale.x *= x;
		fScale.y *= y;
		fScale.z *= z; }

void TxformInfo::SetRotation(float theta) {
		fRotation.SetRotation(theta);
}
void TxformInfo::SetRotation(float x, float y, float z) {
		fRotation.SetRotation(D3DXVECTOR3(x, y, z));
}
void TxformInfo::UpdateRotation(float x, float y, float z) {
		fRotation.UpdateRotation(D3DXVECTOR3(x, y, z));
}
void TxformInfo::SetOrientation(const D3DXMATRIX &m) {
		fRotation.SetOrientation(m);
}

const D3DXMATRIX&
TxformInfo::GetRotationMatrix() const {
		return fRotation.GetRotationMatrix();
}
