#include "TmyWorld.h"
#include "../Resource.h"
#include "../TmodelControl.h"
#include "../Collections/TmeshLibrary.h"
#include "../Collections/TtextureLibrary.h"
#include "../Td3dDrawable.h"
#include "../GraphicsAPI/Tcamera.h"


#define POINT_LIGHT_ID 		2
#define DIRECTION_LIGHT_ID 	1
#define SPOT_LIGHT_ID 		0

TmyWorld::TmyWorld() : 
		fPointLgt(POINT_LIGHT_ID),
		fDirLgt(DIRECTION_LIGHT_ID),
		fSpotLgt(SPOT_LIGHT_ID)
{
	fValidPickRay = false;
	fControl = NULL;
	
	// fCraft - nothing for now ...

	fMainFrame.SetScale(5);

	fGround.SetScale(20, 1, 20);
	fGround.SetLocationTo(0, 0, 10);
	// fGround.SetMeshSurfaceIndex(0);
	fGround.SetTextureIndex(1);
	Tmaterial &mt = fGround.GetMaterial();
	mt.SetDiffuse(0.3);

	
	#define SHADOW_INTENSITY 0.0
	#define SHADOW_BLEND     0.5
	static const D3DXCOLOR sfShadowColor(SHADOW_INTENSITY, SHADOW_INTENSITY, SHADOW_INTENSITY, SHADOW_BLEND);

	fTeapot.SetScale(3, 3, 3);
	fTeapot.SetLocationTo(-5, 2.5, 8);
	fTeapot.SetMeshIndex(eTeapotMesh);
	fTeapot.SetTextureIndex(0);
	Tmaterial &tmt = fTeapot.GetMaterial();
	tmt.SetDiffuse(0.5);
	tmt.SetEmissive(sfShadowColor);

	
	fToyCraft.SetLocationTo(5, 3, 8);
	fToyCraft.SetScale(3, 2, 3);

	fPointLgt.SetLocationTo(10, 10, 10);
	// fPointLgt.DisableVisibility();
	Tmaterial &pmt = fPointLgt.GetMaterial();
	pmt.SetDiffuse(0.5, 0.5, 0.5);
	
	fDirLgt.SetLocationTo(10, 20, 0);
	Tmaterial &dmp = fDirLgt.GetMaterial();
	dmp.SetDiffuse(0.9, 0.7, 0.9);

	fMirror.SetScale(15, 5, 15);
	fMirror.SetRotation(-M_PI/2.0, 0, 0);
	fMirror.SetLocationTo(0, 0, -10);
	fMirror.EnableLighting();
	fMirror.EnableAlphaBlending();
	Tmaterial &bmt = fMirror.GetMaterial();
	bmt.SetEmissive(0);
	bmt.SetDiffuse(0.6, 0.6, 0.6, 0.7);

	// This is our "magic" x-ray light, which shoots out 
	fSpotLgt.SetLocationTo(0, 20, 0);
	fSpotLgt.DisableVisibility();
	Tmaterial &smp = fSpotLgt.GetMaterial();
	smp.SetDiffuse(0.6, 0.2, 0.2);
}

void
TmyWorld::SetControl(
	TmodelControl *c
)
{
	fControl = c;
}
