#ifndef TMY_WORLD
#define TMY_WORLD

#include "./TtoyCraft.h"

#include "../GraphicsAPI/Tmesh.h"
#include "../GraphicsAPI/Trectangle.h"
#include "../GraphicsAPI/TaxisFrame.h"

#include "../GraphicsAPI/TpointLight.h"
#include "../GraphicsAPI/TdirectionalLight.h"
#include "../GraphicsAPI/TspotLight.h"
#include "../GraphicsAPI/Tcamera.h"

#include "TmeshSurface.h"

class TmodelControl;
class TmyWorld {
	private:
		//
		// World model ...
		//
		TtoyCraft			fToyCraft;
		Tmesh				fTeapot;

		Trectangle			fMirror;
		Trectangle			fGround;

		TpointLight			fPointLgt;
		TdirectionalLight	fDirLgt;
		TspotLight			fSpotLgt;

		TaxisFrame			fMainFrame;

		D3DXVECTOR3			fPickRayO, fPickRayD;
		bool				fValidPickRay;

		//
		// Camera 
		Tcamera			*fViewCamera;

		//
		// control
		//
		TmodelControl 	*fControl;

		void RedrawAllObjects(Td3dDrawable&);
		void DrawMirror(Td3dDrawable&);
		void DrawGroundAndShadow(Td3dDrawable&);
		void DrawShadowFromLight(Td3dDrawable&, const D3DXPLANE&, const Tlight&);

	public:
		TmyWorld();

		void UpdateWorld(Td3dDrawable&);
		void DrawWorld(Td3dDrawable*);

		void SetControl(TmodelControl *c);
		void SetViewCamera(Tcamera *c) { fViewCamera = c; }


		void SetPickRay(D3DXVECTOR3 &o, D3DXVECTOR3 &d) {
							fValidPickRay = true;
							fPickRayO = o; 
							fPickRayD = d;}
		void InvalidatePickRay() { fValidPickRay = false; }


		//
		// Getting the objects in the world
		TgraphicsObject *GetWholeToyObject() { return &fToyCraft; }
		TgraphicsObject *GetSupportBaseObject() { return &fToyCraft; }
		TgraphicsObject *GetCraftObject() { return &fToyCraft; /* fSelectedCraft->GetCraft(); */}
		TgraphicsObject *GetTeapotObject() { return &fTeapot; }
		TgraphicsObject *GetMirrorObject() { return &fMirror; }


		TgraphicsObject *GetGroundObject() { return &fGround; }
		TgraphicsObject *GetSpotLightObject() { return &fSpotLgt; }
		TgraphicsObject *GetDirectionalLightObject() { return &fDirLgt; }
		TgraphicsObject *GetPointLightObject() { return &fPointLgt; }

};

#endif
