#include "../resource.h"
#include "../Td3dDrawable.h"
#include "../TmodelControl.h"
#include "TmyWorld.h"

#include "../GraphicsAPI/Tcircle.h"
#include "../GraphicsAPI/Tpoint.h"
#include "../GraphicsAPI/Tline.h"



//
// For debugging picking (set by lowerest geometry
// We will draw in the main routine ...
//
#ifdef PICK_RAW_MESH_DEBUG
	#include "../Collections/TrawMesh.h"
	Tpoint sfWorldSph;
	Tline  sfWorldLine;
	TrawMesh *sfPickedMesh = NULL;
#endif

void TmyWorld::RedrawAllObjects(
	Td3dDrawable &d
)
{
		// now draw the ground ...
		fToyCraft.DrawObject(d);
		fTeapot.DrawObject(d);


		// now draw the lights themselves ...
		fPointLgt.DrawObject(d);
		fDirLgt.DrawObject(d);
		fSpotLgt.DrawObject(d);


		DrawGroundAndShadow(d);
}

void
TmyWorld::DrawWorld(
	Td3dDrawable *d
)
{
	#ifdef DEBUG_RELEASE
		d->fMeshLibrary.PrintLibraryToLog("My World Draw");
	#endif

	fControl->ControlSetObject();

	// Draw the main fame before the model rotation
	//
	
	
	ID3DXMatrixStack *stack = d->Obj2WldStack();
	LPDIRECT3DDEVICE9 device = d->D3DDevice();

	stack->Push();
		//
		// switch on the lights ... only when are visible ...
		//
		if (fPointLgt.VisibilityIsEnabled())
			fPointLgt.SwitchOnLight(device);
		else
			fPointLgt.SwitchOffLight(device);

			
		if (fDirLgt.VisibilityIsEnabled())
			fDirLgt.SwitchOnLight(device);
		else
			fDirLgt.SwitchOffLight(device);

		if (fSpotLgt.VisibilityIsEnabled())
			fSpotLgt.SwitchOnLight(device);
		else
			fSpotLgt.SwitchOffLight(device);


		RedrawAllObjects(*d);
		DrawMirror(*d);

			
	stack->Pop();
	device->SetTransform(D3DTS_WORLD, stack->GetTop());
	
	#ifdef PICK_RAW_MESH_DEBUG
	if (sfPickedMesh != NULL) {
		sfWorldSph.DrawObject(*d);
		sfWorldLine.DrawObject(*d);
		sfPickedMesh->DrawRawMesh(true);
	}
	#endif
}

