// Td3dDrawable.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExample.h"
#include "Td3dDrawable.h"
#include "./GraphicsAPI/Tcamera.h"
#include "TmodelControl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "Model/TmyWorld.h"

/////////////////////////////////////////////////////////////////////////////
// Td3dDrawable

Td3dDrawable::Td3dDrawable()
{
	
	fMyWorld = NULL;
	fD3dDevice = NULL;
	fObj2WorldStack = NULL;
	fMyCamera = NULL;
	fMyControl = NULL;

	ResetMouseRotation();
}

Td3dDrawable::~Td3dDrawable()
{
	if (fD3dDevice != NULL) {
		fD3dDevice->Release();
		fD3dDevice = NULL;
	}
	if (fObj2WorldStack != NULL) {
		fObj2WorldStack->Release();
		fObj2WorldStack = NULL;
	}
}


BEGIN_MESSAGE_MAP(Td3dDrawable, CWnd)
	//{{AFX_MSG_MAP(Td3dDrawable)
	ON_WM_LBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONUP()
	ON_WM_MBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


LPDIRECT3DDEVICE9 Td3dDrawable::D3DDevice(){return fD3dDevice;}
ID3DXMatrixStack* Td3dDrawable::Obj2WldStack(){	return fObj2WorldStack;}
void Td3dDrawable::SetMyWorld(TmyWorld *w){	fMyWorld = w;}
void Td3dDrawable::SetMyCamera(Tcamera *c) { fMyCamera = c; }

//
//
void Td3dDrawable::SetDrawableSize(int w, int h) 
{
	fWidth = w;
	fHeight = h;
	fMyCamera->SetAspectRatio( float(w) / float(h) );
}



/////////////////////////////////////////////////////////////////////////////
// Td3dDrawable message handlers

BOOL Td3dDrawable::PreTranslateMessage(MSG* pMsg) 
{
	return (GetParent())->PreTranslateMessage(pMsg);
}

void Td3dDrawable::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CString msg;
	msg.Format("Begin Zooming: Point(%d %d)", point.x, point.y);
	EchoToStatusArea(msg);

	#define kZoomingDelta  100
	#define kZoomingSize   5.0f

	// Zooming
	// We are only going to pay attention to the x-movement
	fXValuator.InitializeValuator(point.x-kZoomingDelta, point.x+kZoomingDelta, -kZoomingSize, kZoomingSize);

}


void Td3dDrawable::OnMButtonDown(UINT nFlags, CPoint point) 
{
	CString msg;
	msg.Format("Begin Tracking Point(%d %d)", point.x, point.y);
	EchoToStatusArea(msg);

	#define kTrackingDelta 100
	#define kTrackingSize   10

	// Tracking
	// We are only going to pay attention to both the x and y movements
	fXValuator.InitializeValuator(point.x-kTrackingDelta, point.x+kTrackingDelta, -kTrackingSize, kTrackingSize);
	fYValuator.InitializeValuator(point.y-kTrackingDelta, point.y+kTrackingDelta, -kTrackingSize, kTrackingSize);
}

void Td3dDrawable::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if ( nFlags & MK_SHIFT ) {
		// Selecting a point
		CString msg;
		msg.Format("Start selection: (%d %d)", point.x, point.y);
		EchoToStatusArea(msg);

		BuildPickRay(point);

	} else {	
		// Rotating camera ...
		//
		BeginMouseRotation("Rotating Camera", point.x, point.y);
		fXRot = fYRot = 0.0f;
	}		
}

void Td3dDrawable::OnMouseMove(UINT nFlags, CPoint point) 
{
	
	switch (nFlags) {
		case (MK_LBUTTON|MK_SHIFT):
			{
				// Selecting
				// Selecting a point
				CString msg;
				msg.Format("selection current position: (%d %d)", point.x, point.y);
				EchoToStatusArea(msg);

				BuildPickRay(point);
			}
			break;

		case MK_LBUTTON:
			{
				// Rotating camera
				CString m;
				m.Format("Rotating Camera: ViewD(%f)", fMyCamera->GetViewDistance());
				UpdateMouseRotation(m, point.x, point.y);
				fMyCamera->RotateCameraY(-fYRot);
				fMyCamera->RotateCameraX(-fXRot);
				fXRot = 0.0f;
				fYRot = 0.0f;
			}
			break;

		case MK_MBUTTON:
				// tracking
			{
				float dx = fXValuator.GetValue(point.x);
				float dy = fYValuator.GetValue(point.y);
				fMyCamera->SetCameraTrack(dx, dy);
				fXValuator.InitializeValuator(point.x-kTrackingDelta, point.x+kTrackingDelta, -kTrackingSize, kTrackingSize);
				fYValuator.InitializeValuator(point.y-kTrackingDelta, point.y+kTrackingDelta, -kTrackingSize, kTrackingSize);
			}
			break;

		case MK_RBUTTON:
				// zooming
				float dx = fXValuator.GetValue(point.x);
				float d = fMyCamera->GetViewDistance();
				fMyCamera->SetViewDistance( d + dx );
				fXValuator.InitializeValuator(point.x-kZoomingDelta, point.x+kZoomingDelta, -kZoomingSize, kZoomingSize);
			break;
	}
	
	// Update the control sliders if necessary
	fMyControl->CameraSetControl();
}

void Td3dDrawable::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if ( nFlags & MK_SHIFT ) {
		// Selecting
		EchoToStatusArea("Done Selection");
		fMyWorld->InvalidatePickRay();
	} else {	
		// Rotating camera
		EchoToStatusArea("Done Rotating Camera");
	}
}

void Td3dDrawable::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// zooming
	EchoToStatusArea("Done Zooming camera ...");
}

void Td3dDrawable::OnMButtonUp(UINT nFlags, CPoint point) 
{
	EchoToStatusArea("Done tracking camera");
}
