//
// D3dDrawable_Draw.cpp : This is the main drawing routine
//

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#include "StdAfx.h"
#include "Td3dDrawable.h"
#include "Model/TmyWorld.h"

//
// The followings are from our GraphicsAPI ...
//
#include "./GraphicsAPI/GraphicsAPI.h"
#include "./GraphicsAPI/Tcamera.h"

void Td3dDrawable::RedrawDrawable() 
{
	if (fD3dDevice == NULL)
		return;

	//*********************************************************************
	//
	// The first thing: Set up transform ...
	//

	{
		//
		// Set up object to world to be identity for now
		//
		D3DXMATRIX obj2world;
		D3DXMatrixIdentity(&obj2world);
		fD3dDevice->SetTransform(D3DTS_WORLD, &obj2world);
	}

	//
	// Define current view ...
	//
	fMyCamera->DefineViewAndProjection(fD3dDevice);
	

	DrawableSpecificInitDraw();
	
	//
	//***************************************************************************************************
	//
	// Now get ready to draw ...
	//
	D3DCOLOR bgColor = D3DCOLOR_COLORVALUE(0.7, 0.7, 0.7, 0);

	fD3dDevice->Clear(0,							// number of rectangles, 0 means the entire area
						NULL,						// rectangles to be cleared, NULL means the entire area
						D3DCLEAR_TARGET|			// clear target (color) 
						D3DCLEAR_ZBUFFER|			//    z-buffer
						D3DCLEAR_STENCIL,			// and **** NEW ****  stencil
						bgColor,					// new color to clear to
						1.0f,						// new Z values to clear to
						0L);							// new Stencil buffer values to clear to 
													// clear stencil buffer to all 0
		fD3dDevice->BeginScene();

			DrawableSpecificDraw();

			if (fMyWorld != NULL)
				fMyWorld->DrawWorld(this);

		fD3dDevice->EndScene();

	fD3dDevice->Present(NULL, NULL, NULL, NULL);

	DrawableSpecificRestoreDraw();
}


void 
Td3dDrawable::ResetMouseRotation() 
{
	fXRot = fYRot = 0.0f;
}

void 
Td3dDrawable::BeginMouseRotation(const CString &inMsg, int x, int y)
{
	fXPos = x; 
	fYPos = y;
	EchoToStatusArea(inMsg);
}

void 
Td3dDrawable::UpdateMouseRotation(const CString &inMsg, int x, int y) 
{
	fXRot += -(y - fYPos) * 0.005f;
	fYRot +=  (x - fXPos) * 0.005f;
	fXPos = x;
	fYPos = y;

	CString msg;
	
	msg.Format("%s: x(%f) y(%f)", inMsg, fXRot*180/M_PI, fYRot*180/M_PI);
	EchoToStatusArea(msg);
}
